| Safe Haskell | None |
|---|---|
| Language | Haskell98 |
Dhall.Yaml
Synopsis
- data Options = Options {}
- parseDocuments :: Parser Bool
- parseQuoted :: Parser Bool
- defaultOptions :: Options
- dhallToYaml :: Options -> Text -> Text -> IO ByteString
Documentation
parseQuoted :: Parser Bool Source #
Arguments
| :: Options | |
| -> Text | Describe the input for the sake of error location. |
| -> Text | Input text. |
| -> IO ByteString |
Convert a piece of Text carrying a Dhall inscription to an equivalent YAML ByteString